replaceall = require 'replaceall'
semver = require 'semver'
path = require 'path'
fs = require 'fs'
_ = require 'underscore'

argv = process.argv[2 ...]

store_dir = path.join process.env[if process.platform == 'win32' then 'USERPROFILE' else 'HOME'], '.dvm'
store_path = "#{store_dir}/packages.json"

current_package_path = 'package.json'

upgrade_only = not ('--force-change' in argv or '-U' in argv)
indent4 = '--indent-4' in argv or '-4' in argv
overwrite_wildcards = not ('--ignore-wildcards' in argv or '-O' in argv)
exists_only = not ('--write-all' in argv or '-E' in argv)

indent = if indent4 then '    ' else '  '

unless fs.existsSync store_dir
  fs.mkdirSync store_dir

unless fs.existsSync store_path
  fs.writeFileSync store_path, JSON.stringify {}

loadSavedVersion = ->
  return JSON.parse fs.readFileSync store_path

loadCurrentPackageInfo = ->
  try
    return JSON.parse fs.readFileSync current_package_path
  catch e
    return null

writeJSONFile = (path, data) ->
  data = JSON.stringify(data, null, indent) + '\n'
  fs.writeFileSync path, data

filterSpecifiedPackages = (packages) ->
  specified_packages = argv[1 ...]

  specified_packages = _.reject specified_packages, (package_name) ->
    return package_name[0] == '-'

  if _.isEmpty(specified_packages) or '@all' in specified_packages
    return packages
  else
    result = {}

    for package_name, version of packages
      if package_name in specified_packages
        result[package_name] = version

    return result

cleanVersion = (version) ->
  return version unless version

  symbols = ['<', '>', '=', '~', '^']

  for symbol in symbols
    version = replaceall symbol, '', version

  return semver.clean version

switch _.first argv
  when 'list'
    for package_name, version of filterSpecifiedPackages loadSavedVersion()
      console.log "#{package_name}@#{version}"

  when 'import'
    current_package_info = loadCurrentPackageInfo()

    current_dependencies = current_package_info.devDependencies ? {}
    current_dependencies = filterSpecifiedPackages _.extend current_dependencies, current_package_info.dependencies

    saved_versions = loadSavedVersion()

    for package_name, version of current_dependencies
      cleaned_version = cleanVersion version

      can_be_save = do ->
        unless cleaned_version
          return false

        unless saved_versions[package_name]
          return true

        if saved_versions[package_name] == cleaned_version
          return false

        unless upgrade_only
          return true

        if upgrade_only and semver.gt cleaned_version, saved_versions[package_name]
          return true

        return false

      if can_be_save
        console.log "Saved #{package_name}@#{cleaned_version}"
        saved_versions[package_name] = cleaned_version

    writeJSONFile store_path, saved_versions

  when 'export'
    current_package_info = loadCurrentPackageInfo()

    unless current_package_info
      console.log 'No exists package.json, auto generated'
      current_package_info =
        dependencies: {}

    packages_to_write = filterSpecifiedPackages loadSavedVersion()

    for package_name, version of packages_to_write
      original_version = current_package_info.dependencies[package_name]
      cleaned_version = cleanVersion original_version

      can_be_write = do ->
        unless original_version
          if exists_only
            return false
          else
            return true

        if version == cleaned_version
          return false

        if overwrite_wildcards and original_version == '*'
          return true

        unless cleaned_version
          return false

        if upgrade_only
          unless semver.gt version, cleaned_version
            return false

        return true

      if can_be_write
        console.log "Write #{package_name}@#{version}"
        current_package_info.dependencies[package_name] = "^#{version}"

    writeJSONFile current_package_path, current_package_info

  else
    console.log fs.readFileSync(path.join __dirname, 'usages.txt').toString()
